/*
 * Vector.java
 *
 * version: 22nd October 2008, 15:11
 * author: yanas
 */
package cz.yanas.pb162.vector;

/**
 * Represents a vector in any n-dimensional space (for n >= 2).
 *
 * @author yanas
 */
public final class Vector {

    /** an array of coordinates that will be initialized in the following constructor */
    private final double[] coordinates;

    /**
     * Constructs an instance of the Vector class using the given dimension.
     * 
     * @param dimension the dimension of the vector space
     */
    public Vector(int dimension) {
        // the dimension should be at least two
        assert (dimension >= 2);

        // initialize the coordinates by creating an array of the given dimension
        coordinates = new double[dimension];
    }

    /**
     * Returns the dimension of the vector space.
     *
     * @return the dimension of the vector space
     */
    public int getDimension() {
        // simply return the lenght of the array storing the coordinates
        return coordinates.length;
    }

    /**
     * Returns the value of the coordinate with the given ordinal.
     *
     * @param ordinal the ordinal of the coordinate
     *
     * @return the value of the coordinate with the given ordinal
     */
    public double getCoordinate(int ordinal) {
        // the ordinal should be between one and the stored dimension (both inclusive)
        assert (ordinal >= 1);
        assert (ordinal <= getDimension());

        // simply return the value of the coordinate
        return coordinates[ordinal - 1];
    }

    /**
     * Sets a new value of the coordinate with the given ordinal.
     *
     * @param ordinal the ordinal of the coordinate
     * @param value the new value of the coordinate
     */
    public void setCoordinate(int ordinal, double value) {
        // the ordinal should be between one and the stored dimension (both inclusive)
        assert (ordinal >= 1);
        assert (ordinal <= getDimension());

        // simply change the value
        coordinates[ordinal - 1] = value;
    }

    /**
     * Calculates and returns the magnitude of the vector.
     *
     * @return the magnitude of the vector
     */
    public double magnitude() {
        // we are about to sum the squares of coordinates, we will start with zero
        double sumOfSquares = 0.0;

        // for each coordinate of the vector
        for (double coordinate : coordinates) {
            // add the square of the value of the current coordinate to the sum
            sumOfSquares += coordinate * coordinate;
        }

        // finally return the square root of the calculated sum
        return Math.sqrt(sumOfSquares);
    }

    /**
     * Returns true if the given object is equal to this instance, false otherwise.
     *
     * @param object an object to be compared with this instance
     *
     * @return true if the given object is equal to this instance
     */
    @Override
    public boolean equals(Object object) {
        // return true if the object is identical to this instance
        if (object == this) {
            return true;
        }

        // return false if the object is not an instance of the Vector class
        if (!(object instanceof Vector)) {
            return false;
        }

        // cast the object to an instance of the Vector class
        Vector vector = (Vector) object;

        // return false if the dimensions are not the same
        if (getDimension() != vector.getDimension()) {
            return false;
        }

        // now loop through all the coordinates and compare them one by one
        for (int i = 1; i <= getDimension(); i++) {
            // return false if the vectors differ at the i-th coordinate
            if (getCoordinate(i) != vector.getCoordinate(i)) {
                return false;
            }
        }

        // return true, the vectors are equal
        return true;
    }

    /**
     * Returns the hash code of this instance.
     *
     * @return the hash code of this instance
     */
    @Override
    public int hashCode() {
        // set some prime number as an initial hash code
        int hashCode = 17;

        // the hash code should be reflect all the coordinates so loop through all of them
        for (double coordinate : coordinates) {
            // convert the value of the coordinate to long and add it to the hash code
            long longBits = Double.doubleToLongBits(coordinate);
            hashCode += 37 * hashCode + (int) (longBits ^ (longBits >>> 32));
        }

        // finally return the hash code
        return hashCode;
    }

    /**
     * Returns the string representation of this instance.
     *
     * @return the string representation of this instance
     */
    @Override
    public String toString() {
        // create a string builder to make concatenating in a loop more effective
        StringBuilder builder = new StringBuilder();

        // again, for each coordinate
        for (double coordinate : coordinates) {
            // append the value of the coordinate and ", " to the string
            builder.append(coordinate + ", ");
        }

        // return the appropriate substring of the string builder enclosed in '(' and ')'
        return ("(" + builder.substring(0, builder.length() - 2) + ")");
    }

}
